module numtest

  !This file contains all subroutines for numerical tests
  ! (see section 6 of the paper)
  !
  !- example 1 
  !- Hankel matrix
  !- Brown matrix
  !- Helsing matrix
  !- rand0  creates a rand vector
  !- vecrand creates two rand vectors 
implicit none
contains

subroutine example1(n)
! example1.f90
! This file creates a matrix defined by :
!  A(i,j) = 1           if i<=j
!           1+j*epsilon else
!
! We store this matrix in matrixfile.dat

integer ::i,j,n,m,init(4)
real(kind=8) :: epsilon 
real(kind=8), allocatable, dimension(:,:) :: A

!open(unit=20,file='inputfile.dat',status='old')
!read(20,*)init
!n=init(1)
!close(20)
m=n
print*,'Enter epsilon : '
read*,epsilon
allocate(A(n,n))
do i=1,n
   do j=1,m
      if (i<=j) then
         A(i,j)=1
      else
         A(i,j)=1+j*epsilon
      end if
   end do
end do

open(unit=10,file='matrixfile.dat',status='unknown')
do i=1,n
    write(10,*) (A(i,j),j=1,n)
end do
close(10)
deallocate(A)
end subroutine example1

subroutine hankel(n)
! hankel.f90                                                                
! This file creates a matrix defined by :                                     
!  A(i,j) = 0.5/(n-i-j+1.5)                                       
!                                                                              
! We store this matrix in matrixfile.dat                                       

integer ::i,j,n,m,init(4)
real(kind=8), allocatable, dimension(:,:) :: A

!open(unit=20,file='inputfile.dat',status='old')
!read(20,*)init
!n=init(1)
!close(20)
m=n
allocate(A(n,n))
do i=1,n
   do j=1,m
      A(i,j)=0.5/(real(n-i-j)+1.5)
   end do
end do

open(unit=10,file='matrixfile.dat',status='unknown')
do i=1,n
    write(10,*) (A(i,j),j=1,n)
end do
close(10)
deallocate(A)
end subroutine hankel

subroutine brown(n)
! brown.f90                                                                  
! This file creates a tridiagonal matrix defined by :                         
!  A(i,i-1) = -1
!  A(i,i)   = epsilon
!  A(i,i+1) = 1                                                               
!                                                                             
! We store this matrix in matrixfile.dat                                        
integer ::i,j,n,m,init(4)
real(kind=8) :: epsilon 
real(kind=8), allocatable, dimension(:,:) :: A

!open(unit=20,file='inputfile.dat',status='old')
!read(20,*)init
!n=init(1)
!close(20)
m=n
print*,'Enter epsilon : '
read*,epsilon
allocate(A(n,n))
do i=1,n
   do j=1,m
      if (i==j)then
         A(i,j)=epsilon 
      else if (j==i+1) then
         A(i,j)=1
      else if (j==i-1) then
         A(i,j)=-1
      else
         A(i,j)=0
      end if
   end do
end do
open(unit=10,file='matrixfile.dat',status='unknown')
do i=1,n
    write(10,*) (A(i,j),j=1,n)
end do
close(10)
deallocate(A)
end subroutine brown


subroutine helsing(n)
! helsing.f90
! This file creates a matrix defined by :
! A(i,j)= -log|z(i)-z(j)| , if i .neq. j
!         -log|r(i)|
! where z(i) are n somehow randomly distributed points in a unit square centered at the origin in the complex plane and where each r(i) is a number in (0,d(i)[, d(i) being the distance between the point z(i) and its nearest neighbour.
! For more details, see
! S. Duminil, A parallel implementation of the CMRH method for dense linear systems, Numer. Algor., DOI : 10.1007/s11075-012-9616-4
!
! We store this matrix in matrixfile.dat
! 01 February 2013
! S. Duminil

integer :: m,n,coef,d,i,j,init(4)
real(kind=8) :: l,r
real(kind=8), allocatable, dimension(:) :: xx,yy,value,value2
real(kind=8), allocatable, dimension(:,:) :: A

m=n
allocate(value(2*m),value2(m),xx(m),yy(m),A(n,n))
open(unit=30,file='vecrand.dat',status='old')
read(30,*)value
do i=1,m
xx(i)=value(2*i-1)
yy(i)=value(2*i)
end do
close(30)
open(unit=40,file='rand0.dat',status='old')
read(40,*)value2
close(40)
coef=-1
do i=1,n
d=0
do j=1,m
r=dsqrt((xx(i)-xx(j))**2+(yy(j)-yy(i))**2)
if (r>0) then
A(i,j)=coef*log(r)
if (d.eq.0)then
d=j
l=dble(0.5*value2(i)*dble(d))
A(i,i)=-log(l)
end if
end if
end do
end do
deallocate(xx,yy,value,value2)
open(unit=10,file='matrixfile.dat',status='unknown')
do i=1,n
write(10,*) (A(i,j),j=1,n)
end do
close(10)
deallocate(A)
end subroutine helsing

subroutine rand0(n)
integer n,i,init(4)
real(kind=8),allocatable,dimension(:) ::x
integer*4,dimension(3)  ::timearray
real                     ::rand
call itime(timearray)     ! Get the current time
i = rand ( timearray(1)+timearray(2)+timearray(3) )
close(20)
allocate(x(n))
do i=1,n
x(i)=rand(0);
enddo
open(unit=10,file='rand0.dat',status='unknown')
do i=1,n
write(unit=10,FMT=*) x(i)
enddo
close(10)
deallocate(x)
end subroutine rand0

subroutine vecrand(n)
! vecrand.f90
! This file creates two random vectors and stores them in the file vecrand.dat
!
! 01 February 13
! Created by S.Duminil
integer n,i,j,a,b,init(4)
real(kind=8),allocatable,dimension(:) ::x,y
integer*4,dimension(3)  ::timearray
real                     ::rand
call itime(timearray)     ! Get the current time
i = rand ( timearray(1)+timearray(2)+timearray(3) )

a=-1
b=1
!do while(a.ge.b)
!print*,'Enter the interval limits'
!read*,a,b
!if (a.ge.b) then
!print*,'Choose a<b'
!endif
!end do
allocate(x(n),y(n))
do i=1,n
x(i)=rand(0)*dble(b-a)-dble(b);
y(i)=rand(0)*dble(b-a)-dble(b);
enddo
open(unit=10,file='vecrand.dat',status='unknown')
do i=1,n
write(unit=10,FMT=*) x(i),y(i)
enddo
close(10)
deallocate(x,y)
end subroutine vecrand



end module numtest
